

// CNT01 - QUAD QUADRATURE ENCODER


// Includes
#include "device.hpp"


// Preprocessor Directives
#pragma once


// Class CNT01
class CNT01 : public Device
{
public:

   // Class Counter
   template <typename T> class Counter;

   // Channels
   static const uint8_t CHANNELS = 4;

   // Modes
   static const uint8_t MODE_NORMAL = 0;
   static const uint8_t MODE_QUADRATURE_1X = 1;
   static const uint8_t MODE_QUADRATURE_2X = 2;
   static const uint8_t MODE_QUADRATURE_4X = 3;

   // Constructor
   CNT01(const IP::Address& address);
   CNT01(const Device& device);

   // Clear Counter
   void clear(void) const;
   void clear(uint8_t channel) const;
   void clear(const std::set<uint8_t>& channel) const;

   // Disable Counter
   void disable(void) const;
   void disable(uint8_t channel) const;
   void disable(const std::set<uint8_t>& channel) const;

   // Enable Counter
   void enable(void) const;
   void enable(uint8_t channel) const;
   void enable(const std::set<uint8_t>& channel) const;

   // Set Mode
   void mode(uint8_t mode) const;
   void mode(uint8_t channel, uint8_t mode) const;
   void mode(const std::set<uint8_t>& channel, uint8_t mode) const;
   void mode(const std::map<uint8_t, uint8_t>& mode) const;

   // Read Counter
   Counter<std::map<uint8_t, int64_t>> read(void) const;
   Counter<int64_t> read(uint8_t channel) const;
   Counter<std::map<uint8_t, int64_t>> read(const std::set<uint8_t>& channel) const;

private:

   // DCP Message Types
   static const uint16_t _MESSAGE_DISABLE = 0x0700;
   static const uint16_t _MESSAGE_ENABLE = 0x0701;
   static const uint16_t _MESSAGE_READ = 0x0702;
   static const uint16_t _MESSAGE_VALUE = 0x0703;
   static const uint16_t _MESSAGE_CLEAR = 0x0704;
   static const uint16_t _MESSAGE_SET_MODE = 0x0705;
};


// Class Counter
template <typename T> class CNT01::Counter
{
public:

   // Get Value
   const T& value(void) const;

   // Get Time [ms]
   uint64_t time(void) const;

private:

   // Constructor
   Counter(const T& value, const uint64_t& time);

   // Variables
   T _value;
   uint64_t _time;

   // Friends
   friend class CNT01;
};


// Get Value
template <typename T> inline const T& CNT01::Counter<T>::value(void) const
{
   // Return Value
   return _value;
}


// Get Time
template <typename T> inline uint64_t CNT01::Counter<T>::time(void) const
{
   // Return Time
   return _time;
}
