

// ETHERNET 3.0.3


// Includes
#include <map>
#include <set>
#include <string>


// Preprocessor Directives
#pragma once


// Class Ethernet
class Ethernet
{
public:

   // Class Address (in Compliance with IEEE 802.3)
   class Address;

   // Class Exception
   class Exception;

   // Get Adapters
   static const std::set<Ethernet> adapter(void);

   // Get Host Address
   const Address host(void) const;

   // Get Adapter Name
   const std::string name(void) const;

private:

   // Network Adapter Info
   class _Info;

   // Get Network Adapter Info List
   static const std::map<std::string, _Info> _info(void);

   // Constructor
   Ethernet(const std::string& name);

   // Operator <
   bool operator <(const Ethernet& ethernet) const;

   // Variables
   std::string _name;

   // Friends
   friend std::less<Ethernet>;
};


// Class Address (in Compliance with IEEE 802.3)
class Ethernet::Address
{
public:

   // Special Address
   static const Address BROADCAST;

   // Constructor
   Address(void);
   Address(uint8_t b0, uint8_t b1, uint8_t b2, uint8_t b3, uint8_t b4, uint8_t b5);

   // Convert to String
   operator const std::string(void) const;

   // Compare
   bool operator ==(const Address& address) const;
   bool operator !=(const Address& address) const;

   // Get Digit
   uint8_t& operator [](uint8_t n);
   const uint8_t& operator [](uint8_t n) const;

   // Check if broadcast
   bool broadcast(void) const;

private:

   // Variables
   uint8_t _b[6];
};


// Class Exception
class Ethernet::Exception
{
public:

   // Class Internal
   class Internal;

   // Class Network
   class Network;

private:

   // Virtual Function for RTTI
   virtual void _func() {}
};


// Class Internal
class Ethernet::Exception::Internal : public Ethernet::Exception
{
};


// Class Network
class Ethernet::Exception::Network : public Ethernet::Exception
{
};
